package com.holub.io;

import java.io.*;
import java.util.*;
import com.holub.io.Std;
import com.holub.tools.debug.Assert;
// import com.holub.tools.Assert;

/**	Encapsulates I/O to a log file. contains simple wrappers to do
 *  line-level buffering and suck up exceptions that are not errors
 *  in the current context. The Log class encapsulates a <code>Writer</code>
 *	rather than extending it in order to provide more reasonable
 *	exception handling. A <code>Log.Failure</code> object (which is a
 *	<code>RuntimeException</code> is thrown when something goes wrong
 *  rather than an <code>IOException</code>. This way you don't have to
 *  litter up your code with try blocks. The {@link #writer} method is
 *	provided to allow access to the underlying <code>Writer</code> should
 *	you need it, however.
 *	<p>
 *	This class is thread safe.
 **/

public class Log
{
	private Writer					log_file		= null;
	private static /*final*/ String	log_file_name   = "log";
	private boolean					timestamp		= false; 

	public class Failure extends RuntimeException
	{	public Failure(String s){super(s);}
	}

	/*******************************************************************
	 * Create a new Log with the specified name. If a file with that
	 * name exsits, then new lines are just appended to it. The current
	 * time and date is written into the file as well.
	 * <p>
	 * An error message is printed if any exceptions are thrown from
	 * the I/O system, but the program is permitted to run. (In this
	 * case, any subsequent attempts to write to the log are silently
	 * ignored.)
	 *
	 * @param name file to which log messages are sent. If null,
	 *  			log messages go to standard error.
	 * @param error_message_stream. Any error messages that indicate
	 *				problems opening the file are sent here.
	 * 				Use Std.bit_bucket() to suppress error messages.
	 */

	public Log( String name, PrintWriter error_message_stream )
	{	Assert.is_true( name != null );
		Assert.is_true( error_message_stream != null );

		log_file_name = name;
		try
		{	
			error_message_stream.println("Logging to " + name);

			log_file = ( name == null )
					? (Writer)( Std.err() )
					: (Writer)( new BufferedWriter(
							new FileWriter(log_file_name, true /*append*/)) )
					;

			write( "\n#----" + new Date().toString() + "----#\n" );
		}
		catch( IOException e )
		{
			error_message_stream.println("Couldn't open log file: " 
							   + log_file_name
							   + "(" + e.getMessage() + ")" );

			error_message_stream.println("Input will not be logged\n");
		}
	}

	/*******************************************************************
	 * Writes to a previously-opened log file.
	 */

	public Log( Writer log_file )
	{	Assert.is_true( log_file != null );
		this.log_file      = log_file;
		this.log_file_name = "???";
		write( "\n#----" + new Date().toString() + "----#\n" );
	}

	/*******************************************************************
	 * Convenience method. Creates a log called "log" in the directory
	 * specified by the log.file system property. If the log.file
	 * System property (which must be specified with the -D command-line
	 * switch) can't be found of if the file can't be opened, then log output
	 * is silently discarded.
	 */

	public Log()
	{	this( System.getProperty("log.file"), Std.err() );
	}

	/*******************************************************************
	 * All messages logged after this call is made will be prefixed by
	 * the current date and time. (This stamp is in addition to the one
	 * that's output at the top of the file, which effectively indicates
	 * when the log file was opened.)
	 */

	public void timestamp()
	{	timestamp = true;
	}

	/*******************************************************************
	 * Append a string of the indicated length, starting at the indicated
	 * offset, to the log file. All internal buffers are flushed
	 * after writing so that the log file itself will be up to date.
	 * An error message if printed if any exceptions are thrown from
	 * the I/O system.
	 * <p>
	 * A Failure is thrown if any exceptions are thrown from any internal
	 * <code>Writer</code> calls.
	 */

	public void write( String text, int offset, int length )
	{	Assert.is_true( text != null 		  				 	);
		Assert.is_true( offset >= 0	 		  				 	);
		Assert.is_true( 0 <= length && length <= text.length()	);

		if( log_file == null ) return;	// do nothing if create failed	

		try
		{	if( timestamp )
				log_file.write( new Date().toString() + ": " );

			log_file.write( text, offset, length );
			log_file.flush();
		}
		catch( IOException e )
		{	throw new Failure( log_file_name + ": " +
												e.getMessage() );
		}
	}
	/*******************************************************************
	 * Convenience method. Logs the entire string.
	 */
	public synchronized void write( String text )
	{	this.write( text, 0, text.length() );
	}

	/*******************************************************************
	 * Close the log file.
	 * A Failure is thrown if any exceptions are thrown from
	 * the I/O system.
	 */
	public synchronized void close( )
	{	if( log_file == null ) return;	// do nothing if create failed	

		try
		{	log_file.close();
		}
		catch( IOException e )
		{	throw new Failure( log_file_name + ": " +
												e.getMessage() );
		}
	}

	/********************************************************************
	 * Objects of the Log.Writer_accessor are returned from the {@link #writer}
	 * method. They implement the Writer interface, and chain most operations
	 * through to the underlying writer. They provide synchronized access to the
	 * Writer as well. (That is, multiple threads can safely and simultaneously
	 * access both the <code>Log</code> object itself and also any
	 * <code>Writers</code> returned from various <code>writer()</code>
	 * calls on that <code>Log</code> object). The only surprise is the
	 * {@link #close} method, which throws a
	 * java.lang.UnsupportedOperationException. You must close the
	 * encapsulating Log object, not the object returned from {@link #writer}.
	 *
	 * Note that:
	 *	<ul>
	 *	<li>the log file is flushed after every write operation (including
	 * 		single-character writes) to make sure that it's as up-to-date as possible.
	 *	<li>If the <code>timestamp()</code> method of the encapsulating Log
	 *		has been called, output sent to the Log using either String
	 *		version of write() is time stamped, but output sent using
	 *		the character versions of write() are not time stamped.
	 *  </ul>
	 */

	private class Writer_accessor extends Writer
	{
		public void write( int c ) throws IOException
		{	synchronized(Log.this)
			{	log_file.write(c);
				log_file.flush();
			}
		}
		public void write( char[] cbuf ) throws IOException
		{	synchronized(Log.this)
			{	log_file.write(cbuf);
				log_file.flush();
			}
		}
		public void write( char[] cbuf, int off, int len ) throws IOException
		{	synchronized(Log.this)
			{	log_file.write(cbuf,off,len);
				log_file.flush();
			}
		}
		public void write( String str ) throws IOException
		{	Log.this.write(str);
		}
		public void write( String str, int off, int len ) throws IOException
		{	Log.this.write(str,off,len);
		}
		public void flush() throws IOException
		{	synchronized(Log.this)
			{	log_file.flush();
			}
		}
		public void close()
		{	throw new java.lang.UnsupportedOperationException
									("Must close encasulating Log object");
		}
	}

	/** Return a Writer that writes to the current log. This writer is a
	 *  singleton in that, for a given <code>Log</code> object, only one
	 *  Writer is created. All calls to <code>writer()</code> return
	 *  references to the same object.
	 */

	public Writer writer()
	{	if( the_writer == null )
		{	synchronized( this )
			{	if( the_writer == null )
					the_writer = new Writer_accessor();
			}
		}
		return the_writer;
	}

	private static Writer_accessor the_writer;


	/**********************************************************************
	 * A Unit test class.
	 */

	public static class Test
	{	public static void main( String[] args )
		{	
			Log log = new Log( "log.test", Std.err() );

			log.write( "hello world\n" );
			log.write( "xxxhello world\n", 3, 12 );

			log.timestamp();

			log.write( "timestamp now on\n" );
			log.write( "xxxhello world\n", 3, 12 );

			try
			{	
				Writer writer = log.writer();
				writer.write( "output directly to writer\n" );
				writer.write( new char[]{ 'c', 'h', 'a', 'r', ' ', 'o', 'u', 't', '\n'} );
			}
			catch( IOException exception )
			{	exception.printStackTrace();
			}
		}
	}
}
