package com.holub.tools;

import com.holub.tools.debug.Assert;
import java.io.*;

/****************************************************************
 * A simple class to help in testing. Various check() methods
 * are passed a test id, an expected value, and an actual value.
 * The test prints appropriate messages, and keeps track of the
 * total error count for you.
 * For example:
 * <pre>
 * class Test
 * {	public static void main(String[] args);
 *		{
 *			// Create a tester that sends output to standard error, which
 *			// operates in verbose mode if there are any command-line
 *			// arguments.
 *
 * 			Tester t = new Tester( args.length > 0,
 *										com.holub.tools.Std.out() );
 * 			//...
 *
 * 			t.check("test.1", 0,     foo()); 		// check that foo() returns 0.
 * 			t.check("test.2", "abc", bar()); 		// check that bar() returns "abc".
 * 			t.check("test.3", true , cow()); 		// check that cow() returns true
 * 			t.check("test.4", true , dog()==null);	// check that dog() returns null
 *
 * 			//...
 *			System.exit( t.total_errors() );
 *		}
 *	}
 * </pre>
 */

public class Tester
{
	private int		 		  errors = 0;
	private final boolean  	  verbose;
	private final PrintWriter log;

	/**
	 * Create a tester that has the specified behavior and output stream:
	 *
	 * @param verbose	Print messages even if test succedes. (Normally,
	 *					only failures are indicated.)
	 * @param log		if not null, all output is sent here, otherwise
	 *					output is sent to standard error.
	 */
	public Tester( boolean verbose, PrintWriter log )
	{	this.verbose = verbose;
		this.log	 = log;
	}
	/******************************************************************
	 * Check that and expected result of type String is equal to the
	 * actual result.
	 *
	 * @param test_id	String that uniquely identifies this test.
	 * @param expected	the expected result
	 * @param actual	the value returned from the function under test
	 * @return true if the expected and actual parameters matched.
	 */

	public boolean check( String test_id, String expected, String actual)
	{
		Assert.is_true( log != null    , "Tester.check(): log is null" 		);
		Assert.is_true( test_id != null, "Tester.check(): test_id is null"	);

		boolean okay = expected.equals( actual );

		if( !okay || verbose )
		{	
			log.println (  (okay ? "   okay " : "** FAIL ")
						 + "("  + test_id + ")"
						 + " expected: " + expected
						 + " got     : " + actual
					);
		}
		return okay;
	}
	/******************************************************************
	 * Convenience method, compares a string against a StringBuffer
	 */
	public boolean check( String test_id, String expected, StringBuffer actual)
	{	return check( test_id, expected, actual.toString());
	}
	/******************************************************************
	 * Convenience method, compares two doubles
	 */
	public boolean check( String test_id, double expected, double actual)
	{	return check( test_id, "" + expected, "" + actual );
	}
	/******************************************************************
	 * Convenience method, compares two longs
	 */
	public boolean check( String test_id, long expected, long actual)
	{	return check( test_id, "" + expected, "" + actual );
	}
	/******************************************************************
	 * Convenience method, compares two booleans
	 */
	public boolean check( String test_id, boolean expected, boolean actual)
	{	return check( test_id, expected?"true":"false", actual?"true":"false" );
	}
	/******************************************************************
	 * Exit the program, using the total error count as the exit status.
	 */
	public void exit()
	{	System.exit( errors );
	}
}
