package com.holub.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 *	A convenience class that creates a JTextArea and wraps it into
 *	a JScrollPane for you. A Scrollable_JTextArea <i>is</i> a JScrollPane
 *	that <i>contains</i> a JTextArea.
 */

public class Scrollable_JTextArea extends JScrollPane
{
	/******************************************************************
	 *	Create an empty Scrollable_JTextArea with scroll bars created as
	 *  needed, and word-wrapping on in the encapsulate JTextArea. (The
	 *  latter  means that you won't ever get a horizontal scroll bar.
	 *  You can change this behavior by sending a setLineWrap(false)
	 *  message to the Scrollable_JTextArea object.) The default perfered
	 *  size is 250 x 250;
	 */

	private final JTextArea text = new JTextArea();

	public Scrollable_JTextArea( boolean write_only )
	{
		// Set up for line wrap an a vertical scroll bar. Other
		// alternatives could be horizontal and vertical
		// scroll bars and no line wrap.

		text.setLineWrap(true);
		text.setWrapStyleWord(true);

		setViewportView( text );

		setPreferredSize( new Dimension(250, 250) );
		setSize			( new Dimension(250, 250) );
		setBorder		( BorderFactory.createEtchedBorder() );

		if( write_only )
		{
			text.setEditable  ( false );
			text.addFocusListener
			(	new FocusAdapter()
				{	public void focusGained(FocusEvent e)
					{	Scrollable_JTextArea.this.requestFocus();
					}
				}
			);
		}
	}

	/******************************************************************
	 * Convenience method, sets the initial text to initial_text.
	 */
	public Scrollable_JTextArea( String initial_text, boolean write_only )
	{	this(write_only);
		text.setText( initial_text );
	}

	/******************************************************************
	 * Convenience method, sets the initial text to initial_text, one
	 * String per line. The initial font is 12-point Monospaced.
	 * The size is controlled by the size of the text. The number of
	 * columns is the maximum number of characters on a line in
	 * initial-text; the number of rows is the number of lines.
	 * Word wrapping is off.
	 */

	public Scrollable_JTextArea( String[] initial_text, boolean write_only )
	{	this(write_only);
		text.setLineWrap( false				  	 				 );
		text.setFont	( new Font("Monospaced", Font.PLAIN, 12) );
		text.setRows	( initial_text.length    				 );

		int width  = 0;
		for( int i = 0 ; i < initial_text.length; ++i )
		{	String current = initial_text[i];
			width = Math.max( width, current.length() );
			text.append( current + "\n" );
		}

		text.setColumns( width );

		Dimension size = text.getPreferredSize();
		size.height += 5;	// Add some slop so that scroll bars won't
		size.width  += 5;	// be displayed initially.

		setPreferredSize( size );
		setSize			( size );
	}

	/******************************************************************
	 * Convenience methods, just chain through to the
	 * setText() method of the contained JTextArea();
	 */

	public final void setText( String str )
	{	text.setText( str );
	}

	public final void append( String str )
	{	text.append( str );
	}

	public final void setEditable(boolean on)
	{	text.setEditable(on); 
	}

	public final void setLineWrap(boolean on)
	{	text.setLineWrap(on);
	}

	public final String getText()
	{	return text.getText();
	}

	public final void setBackground(Color c)
	{  if(text!=null) text.setBackground(c);
	}

	public final void setForeground(Color c)
	{ if(text!=null) text.setForeground(c);
	}

	public final void setFont(Font f)
	{ if(text!=null) text.setFont(f);
	}

	/******************************************************************
	 * Accessor method, provides access to the contained JTextArea
	 * if you need it.
	 */
	public final JTextArea getTextArea( ){ return text;	}

	/******************************************************************
	 * Container of the unit-test
	 */
	static public class Test
	{
		static public void main( String[] args )
		{	test1();
			test2();
		}

		static void test1()
		{
			Scrollable_JTextArea text = new Scrollable_JTextArea(
			        "This is an editable JTextArea " +
			        "that has been initialized with the setText method. " +
			        "A text area is a \"plain\" text component, " +
			        "which means that although it can display text " +
			        "in any font, all of the text is in the same font.",
					false
			);

			JFrame frame = new JFrame();
			frame.getContentPane().add( text );
			frame.setVisible( true );

			try{ Thread.currentThread().sleep(10000); } catch (Exception e){}

			text.setText(	"Here is a completely new\n"+
							"chunk of text\n"+
						  	"this time\n" +
							"with newlines."
						);
		}

		static void test2()
		{	String[] contents = { "This is a read-only ->",
								  "multiline text control ->",
								  "that should display->",
								  "four lines.->"
								};

			Scrollable_JTextArea text = new Scrollable_JTextArea( contents,true );
			JFrame frame = new JFrame();
			frame.getContentPane().add( text );
			frame.pack();
			frame.setVisible( true );
		}
	}
}
